#!/usr/bin/env python3

import logging
import shutil
import sys
from datetime import datetime
from os import environ

import requests
import tomlkit
import urllib3
from mako.template import Template

urllib3.disable_warnings()

CONFIGFILE = environ.get('STATUSPAGE_CONFIG', 'config.toml')


class StatusPage:
    def get_api_result(self):
        if self.services:
            log.debug('services already exist, returning early')

            return self.services

        headers = {'Accept': 'application/json', 'X-HTTP-Method-Override': 'GET'}

        requestbody = {
            "attrs": [
                "name",
                "state",
                "last_check_result",
                "host_name",
                "display_name",
            ],
            "joins": ["host", "host.state", "host.last_check_result", "host.vars"],
            "filter": self.config['filters']['services'],
        }

        r = requests.get(
            '{}/v1/objects/services'.format(self.config['icinga2_api']['baseurl']),
            headers=headers,
            json=requestbody,
            auth=(
                self.config['icinga2_api']['username'],
                self.config['icinga2_api']['password'],
            ),
            verify=False,
        )

        self.logger.info(f'got http status code {r.status_code}')
        self.logger.debug(r.text)

        if r.status_code == 200:
            self.services = r.json()['results']
        else:
            r.raise_for_status()

        self.logger.info(f'got {len(self.services)} services from api')

        return self.services

    def prettify(self, text):
        for search, replace in self.config.get('prettify', {}).items():
            text = text.replace(search, replace)

        return text

    def get_services_per_host(self):
        state_to_design_mapping = [
            ('success', 'OK'),
            ('warning', 'WARNING'),
            ('danger', 'CRITICAL'),
            ('info', 'UNKNOWN'),
        ]
        result = {}

        for service in self.get_api_result():
            self.logger.info(
                f'now processing {service["attrs"]["host_name"]} "{service["attrs"]["display_name"]}"'
            )
            self.logger.debug(service)

            host = service['joins']['host']['vars']['pretty_name']

            if host not in result:
                result[host] = {
                    'hostname': service['attrs']['host_name'],
                    'services': {},
                }
                if service['joins']['host']['state'] == 0:
                    result[host]['host_badge'] = 'success'
                    result[host]['host_state'] = 'UP'
                else:
                    result[host]['host_badge'] = 'danger'
                    result[host]['host_state'] = 'DOWN'
                    self.ragecounter += 10

            state = int(service['attrs']['state'])

            if state in (1, 2):
                self.ragecounter += state

            result[host]['services'][
                self.prettify(service['attrs']['display_name'])
            ] = {
                'badge': state_to_design_mapping[state][0],
                'state': state_to_design_mapping[state][1],
            }
            self.logger.info(f'ragecounter is now {self.ragecounter}')

        return result

    def render_html(self, service_details):
        if self.ragecounter == 0:
            mood = '🆗'
        elif self.ragecounter < 10:
            mood = '🚨'
        else:
            mood = '🔥'

        self.logger.info('rendering output html')

        start = datetime.now()
        template = Template(
            filename=self.config['output'].get('template', 'template.html')
        )
        output = template.render(
            title=self.config['output'].get('page_title', 'Status Page'),
            mood=mood,
            hosts=service_details,
        )
        end = datetime.now()

        self.logger.info(f'rendered in {(end-start).total_seconds():.09f}s')

        with open(self.config['output']['filename'], 'w') as f:
            f.write(output)

    def __init__(self):
        self.config = tomlkit.loads(open(CONFIGFILE).read())
        self.services = {}
        self.ragecounter = 0

        self.logger = logging.getLogger('StatusPage')
        handler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter(
            '%(levelname)s {%(filename)s:%(lineno)d} %(message)s'
        )
        handler.setFormatter(formatter)
        self.logger.addHandler(handler)
        self.logger.setLevel(self.config.get('loglevel', 'INFO'))


if __name__ == "__main__":
    page = StatusPage()

    try:
        service_details = page.get_services_per_host()
        page.render_html(service_details)
    except Exception as e:
        shutil.copyfile('error.html', page.config['output']['filename'])
        raise e
